@@ -12,19 +12,27 @@ from pydantic import BaseModel, ValidationError, constr, validator
 from ..utils.logger import payment_logger as logger
 
 
-def check_iso_format(v: str):
-    try:
-        return parser.parse(v)
-    except Exception:
-        raise ValidationError('date must be ISO8601 format')
-
-
-def check_isoformat_none_case(v: Optional[str]):
-    if v is not None:
+def check_iso_format(v):
+    if isinstance(v, str):
         try:
-            return parser.parse(v)
+            return parser.parse(v)  # Converte stringa in datetime
         except Exception:
-            raise ValidationError('date must be ISO8601 format')
+            raise ValidationError("date must be a valid ISO8601 datetime")
+    if isinstance(v, datetime.datetime):
+        return v  # Se è già un datetime, lo lascia invariato
+    raise ValidationError("expire_at must be a datetime or an ISO8601 string")
+
+
+def check_isoformat_none_case(v):
+    if v is not None:
+        if isinstance(v, str):
+            try:
+                return parser.parse(v)  # Converte stringa in datetime
+            except Exception:
+                raise ValidationError("date must be a valid ISO8601 datetime")
+        if isinstance(v, datetime.datetime):
+            return v  # Se è già un datetime, lo lascia invariato
+        raise ValidationError("date must be a datetime or an ISO8601 string")
 
 
 class HttpMethodType(Enum):
@@ -124,8 +132,8 @@ class Receiver(BaseModel):
 
 class PaymentData(BaseModel):
     transaction_id: Optional[str]
-    paid_at: Optional[str]
-    expire_at: Optional[str]
+    paid_at: Optional[datetime.datetime]
+    expire_at: Optional[datetime.datetime]
     amount: Optional[float]
     currency: CurrencyType
     type: Optional[PaymentType]
@@ -142,8 +150,8 @@ class PaymentData(BaseModel):
         validate_all = True
         validate_assignment = True
 
-    paid_at_must_be_iso8601 = validator('paid_at', allow_reuse=True)(check_isoformat_none_case)
-    expire_at_must_be_iso8601 = validator('expire_at', allow_reuse=True)(check_isoformat_none_case)
+    paid_at_must_be_iso8601 = validator('paid_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
+    expire_at_must_be_iso8601 = validator('expire_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
 
     def json(self):
         return json.dumps(self, cls=PaymentEncoder)
@@ -151,10 +159,10 @@ class PaymentData(BaseModel):
 
 class OnlinePaymentBegin(BaseModel):
     url: Optional[str]
-    last_opened_at: Optional[str]
+    last_opened_at: Optional[datetime.datetime]
     method: Optional[HttpMethodType]
 
-    last_opened_at_must_be_iso8601 = validator('last_opened_at', allow_reuse=True)(check_isoformat_none_case)
+    last_opened_at_must_be_iso8601 = validator('last_opened_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
 
     class Config:
         validate_all = True
@@ -187,9 +195,9 @@ class Cancel(OnlinePaymentBegin):
 class Notify(BaseModel):
     url: Optional[str]
     method: Optional[HttpMethodType]
-    sent_at: Optional[str]
+    sent_at: Optional[datetime.datetime]
 
-    sent_at_must_be_iso8601 = validator('sent_at', allow_reuse=True)(check_isoformat_none_case)
+    sent_at_must_be_iso8601 = validator('sent_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
 
     class Config:
         validate_all = True
@@ -201,12 +209,12 @@ class Notify(BaseModel):
 
 class Update(BaseModel):
     url: Optional[str]
-    last_check_at: Optional[str]
-    next_check_at: Optional[str]
+    last_check_at: Optional[datetime.datetime]
+    next_check_at: Optional[datetime.datetime]
     method: Optional[HttpMethodType]
 
-    last_check_at_must_be_iso8601 = validator('last_check_at', allow_reuse=True)(check_isoformat_none_case)
-    next_check_at_must_be_iso8601 = validator('next_check_at', allow_reuse=True)(check_isoformat_none_case)
+    last_check_at_must_be_iso8601 = validator('last_check_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
+    next_check_at_must_be_iso8601 = validator('next_check_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
 
     class Config:
         validate_all = True
@@ -289,8 +297,8 @@ class Payment(BaseModel):
     type: str
     tenant_id: UUID
     service_id: UUID
-    created_at: str
-    updated_at: str
+    created_at: datetime.datetime
+    updated_at: datetime.datetime
     status: PaymentStatus
     reason: Optional[constr(max_length=140)]
     remote_id: UUID
@@ -301,7 +309,7 @@ class Payment(BaseModel):
     locale: str = None
     event_id: UUID
     event_version: str
-    event_created_at: str
+    event_created_at: datetime.datetime
     app_id: str
 
     class Config:
